"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  parseForESLint: () => parseForESLint2
});
module.exports = __toCommonJS(src_exports);

// src/ts.ts
var import_path2 = __toESM(require("path"));
var import_typescript = __toESM(require("typescript"));

// src/transform/index.ts
var import_path = __toESM(require("path"));

// src/transform/vue.ts
function transformForVue(code, context) {
  if (context.current) {
    return code;
  }
  const compiler = require("vue/compiler-sfc");
  const result = compiler.parse(code);
  const compiled = compiler.compileScript(result.descriptor, {
    id: "id",
    reactivityTransform: true
  });
  return compiled.content;
}

// src/transform/svelte.ts
function transformForSvelte(code, context) {
  if (context.current) {
    return code;
  }
  const svelte2tsx = require("svelte2tsx");
  const result = svelte2tsx.svelte2tsx(code, {
    filename: context.filePath
  });
  return result.code;
}

// src/transform/astro.ts
function transformForAstro(code, context) {
  if (context.current) {
    return code;
  }
  const compiler = require("astrojs-compiler-sync");
  const result = compiler.convertToTSX(code, {
    sourcefile: context.filePath
  });
  return result.code;
}

// src/transform/index.ts
function transformExtraFile(code, context) {
  const ext = import_path.default.extname(context.filePath);
  const transform = ext === ".vue" ? transformForVue : ext === ".svelte" ? transformForSvelte : ext === ".astro" ? transformForAstro : () => code;
  try {
    return transform(code, context);
  } catch (e) {
  }
  return code;
}

// src/ts.ts
var TSServiceManager = class {
  constructor() {
    this.tsServices = /* @__PURE__ */ new Map();
  }
  getProgram(code, options) {
    const tsconfigPath = options.project;
    const extraFileExtensions = [...new Set(options.extraFileExtensions)];
    let serviceList = this.tsServices.get(tsconfigPath);
    if (!serviceList) {
      serviceList = [];
      this.tsServices.set(tsconfigPath, serviceList);
    }
    let service = serviceList.find(
      (service2) => extraFileExtensions.every(
        (ext) => service2.extraFileExtensions.includes(ext)
      )
    );
    if (!service) {
      service = new TSService(tsconfigPath, extraFileExtensions);
      serviceList.unshift(service);
    }
    return service.getProgram(code, options.filePath);
  }
};
var TSService = class {
  constructor(tsconfigPath, extraFileExtensions) {
    this.currTarget = {
      code: "",
      filePath: "",
      dirMap: /* @__PURE__ */ new Map()
    };
    this.fileWatchCallbacks = /* @__PURE__ */ new Map();
    this.tsconfigPath = tsconfigPath;
    this.extraFileExtensions = extraFileExtensions;
    this.watch = this.createWatch(tsconfigPath, extraFileExtensions);
  }
  getProgram(code, filePath) {
    var _a;
    const normalized = normalizeFileName(
      toRealFileName(filePath, this.extraFileExtensions)
    );
    const lastTarget = this.currTarget;
    const dirMap = /* @__PURE__ */ new Map();
    let childPath = normalized;
    for (const dirName of iterateDirs(normalized)) {
      dirMap.set(dirName, { path: childPath, name: import_path2.default.basename(childPath) });
      childPath = dirName;
    }
    this.currTarget = {
      code,
      filePath: normalized,
      dirMap
    };
    for (const { filePath: targetPath } of [this.currTarget, lastTarget]) {
      if (!targetPath)
        continue;
      if (!import_typescript.default.sys.fileExists(targetPath)) {
        (_a = this.fileWatchCallbacks.get(normalizeFileName(this.tsconfigPath))) == null ? void 0 : _a();
      }
      getRefreshTargetFileNames(targetPath, this.extraFileExtensions).forEach(
        (vFilePath) => {
          var _a2;
          (_a2 = this.fileWatchCallbacks.get(vFilePath)) == null ? void 0 : _a2();
        }
      );
    }
    const program = this.watch.getProgram().getProgram();
    program.getTypeChecker();
    return program;
  }
  createWatch(tsconfigPath, extraFileExtensions) {
    const normalizedTsconfigPaths = /* @__PURE__ */ new Set([normalizeFileName(tsconfigPath)]);
    const watchCompilerHost = import_typescript.default.createWatchCompilerHost(
      tsconfigPath,
      {
        noEmit: true,
        jsx: import_typescript.default.JsxEmit.Preserve,
        allowNonTsExtensions: true
      },
      import_typescript.default.sys,
      import_typescript.default.createAbstractBuilder,
      (diagnostic) => {
        throw new Error(formatDiagnostics([diagnostic]));
      },
      () => {
      },
      void 0
    );
    const original = {
      readFile: watchCompilerHost.readFile,
      fileExists: watchCompilerHost.fileExists,
      readDirectory: watchCompilerHost.readDirectory,
      directoryExists: watchCompilerHost.directoryExists,
      getDirectories: watchCompilerHost.getDirectories
    };
    watchCompilerHost.getDirectories = (dirName, ...args) => {
      var _a;
      return distinctArray(
        ...original.getDirectories.call(watchCompilerHost, dirName, ...args),
        (_a = this.currTarget.dirMap.get(normalizeFileName(dirName))) == null ? void 0 : _a.name
      );
    };
    watchCompilerHost.directoryExists = (dirName, ...args) => {
      return original.directoryExists.call(watchCompilerHost, dirName, ...args) || this.currTarget.dirMap.has(normalizeFileName(dirName));
    };
    watchCompilerHost.readDirectory = (dirName, ...args) => {
      const results = original.readDirectory.call(
        watchCompilerHost,
        dirName,
        ...args
      );
      const file = this.currTarget.dirMap.get(normalizeFileName(dirName));
      if (file && file.path === this.currTarget.filePath) {
        results.push(file.path);
      }
      return distinctArray(...results).map((result) => {
        if (!isExtra(result, extraFileExtensions)) {
          return result;
        }
        if (original.fileExists.call(watchCompilerHost, `${result}.d.ts`)) {
          return result;
        }
        return toVirtualTSXFileName(result, extraFileExtensions);
      });
    };
    watchCompilerHost.readFile = (fileName, ...args) => {
      const realFileName = toRealFileName(fileName, extraFileExtensions);
      const normalized = normalizeFileName(realFileName);
      if (this.currTarget.filePath === normalized) {
        return transformExtraFile(this.currTarget.code, {
          filePath: normalized,
          current: true
        });
      }
      const code = original.readFile.call(
        watchCompilerHost,
        realFileName,
        ...args
      );
      if (!code) {
        return code;
      }
      if (normalizedTsconfigPaths.has(normalized)) {
        const configJson = import_typescript.default.parseConfigFileTextToJson(realFileName, code);
        if (!configJson.config) {
          return code;
        }
        if (configJson.config.extends) {
          for (const extendConfigPath of [configJson.config.extends].flat()) {
            normalizedTsconfigPaths.add(
              normalizeFileName(
                toAbsolutePath(extendConfigPath, import_path2.default.dirname(normalized))
              )
            );
          }
        }
        if (!configJson.config.include) {
          return code;
        }
        const include = [configJson.config.include].flat().map((s) => toVirtualTSXFileName(s, extraFileExtensions));
        return JSON.stringify(__spreadProps(__spreadValues({}, configJson.config), {
          include
        }));
      }
      return transformExtraFile(code, {
        filePath: normalized,
        current: false
      });
    };
    watchCompilerHost.fileExists = (fileName, ...args) => {
      const normalizedFileName = normalizeFileName(fileName);
      if (this.currTarget.dirMap.has(normalizedFileName)) {
        return false;
      }
      const normalizedRealFileName = toRealFileName(
        normalizedFileName,
        extraFileExtensions
      );
      if (this.currTarget.filePath === normalizedRealFileName) {
        return true;
      }
      const real = toRealFileName(fileName, extraFileExtensions);
      if (original.fileExists.call(watchCompilerHost, real, ...args)) {
        if (real !== fileName) {
          if (original.fileExists.call(watchCompilerHost, `${real}.d.ts`, ...args)) {
            return false;
          }
        }
        return true;
      }
      return false;
    };
    watchCompilerHost.watchFile = (fileName, callback) => {
      const normalized = normalizeFileName(fileName);
      this.fileWatchCallbacks.set(
        normalized,
        () => callback(fileName, import_typescript.default.FileWatcherEventKind.Changed)
      );
      return {
        close: () => {
          this.fileWatchCallbacks.delete(normalized);
        }
      };
    };
    watchCompilerHost.watchDirectory = () => {
      return {
        close: () => {
        }
      };
    };
    watchCompilerHost.afterProgramCreate = (program) => {
      const originalDiagnostics = program.getConfigFileParsingDiagnostics();
      const configFileDiagnostics = originalDiagnostics.filter(
        (diag) => diag.category === import_typescript.default.DiagnosticCategory.Error && diag.code !== 18003
      );
      if (configFileDiagnostics.length > 0) {
        throw new Error(formatDiagnostics(configFileDiagnostics));
      }
    };
    const watch = import_typescript.default.createWatchProgram(watchCompilerHost);
    return watch;
  }
};
function getRefreshTargetFileNames(fileName, extraFileExtensions) {
  if (isExtra(fileName, extraFileExtensions)) {
    return [`${fileName}.tsx`, `${fileName}.d.ts`, fileName];
  }
  return [fileName];
}
function toVirtualTSXFileName(fileName, extraFileExtensions) {
  if (isExtra(fileName, extraFileExtensions)) {
    return `${fileName}.tsx`;
  }
  return fileName;
}
function toRealFileName(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(`${extraFileExtension}.tsx`)) {
      return fileName.slice(0, -4);
    }
  }
  return fileName;
}
function isExtra(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(extraFileExtension)) {
      return true;
    }
  }
  return false;
}
function formatDiagnostics(diagnostics) {
  return import_typescript.default.formatDiagnostics(diagnostics, {
    getCanonicalFileName: (f) => f,
    getCurrentDirectory: () => process.cwd(),
    getNewLine: () => "\n"
  });
}
function normalizeFileName(fileName) {
  let normalized = import_path2.default.normalize(fileName);
  if (normalized.endsWith(import_path2.default.sep)) {
    normalized = normalized.slice(0, -1);
  }
  if (import_typescript.default.sys.useCaseSensitiveFileNames) {
    return toAbsolutePath(normalized, null);
  }
  return toAbsolutePath(normalized.toLowerCase(), null);
}
function toAbsolutePath(filePath, baseDir) {
  return import_path2.default.isAbsolute(filePath) ? filePath : import_path2.default.join(baseDir || process.cwd(), filePath);
}
function* iterateDirs(filePath) {
  let target = filePath;
  let parent;
  while ((parent = import_path2.default.dirname(target)) !== target) {
    yield parent;
    target = parent;
  }
}
function distinctArray(...list) {
  return [
    ...new Set(
      import_typescript.default.sys.useCaseSensitiveFileNames ? list : list.map((s) => s == null ? void 0 : s.toLowerCase())
    )
  ].filter((s) => s != null);
}

// src/index.ts
var tsEslintParser = __toESM(require("@typescript-eslint/parser"));
var DEFAULT_EXTRA_FILE_EXTENSIONS = [".vue", ".svelte", ".astro"];
var tsServiceManager = new TSServiceManager();
function parseForESLint2(code, options = {}) {
  if (!options.project) {
    return tsEslintParser.parseForESLint(code, options);
  }
  const extraFileExtensions = options.extraFileExtensions || DEFAULT_EXTRA_FILE_EXTENSIONS;
  const programs = [];
  for (const option of iterateOptions(options)) {
    programs.push(tsServiceManager.getProgram(code, option));
  }
  let filePath = options.filePath;
  if (filePath && extraFileExtensions.some((ext) => {
    var _a;
    return (_a = options.filePath) == null ? void 0 : _a.endsWith(ext);
  })) {
    filePath = `${filePath}.tsx`;
  }
  const parserOptions = __spreadProps(__spreadValues({}, options), {
    filePath,
    programs,
    extraFileExtensions
  });
  return tsEslintParser.parseForESLint(code, parserOptions);
}
function* iterateOptions(options) {
  if (!options) {
    throw new Error("`parserOptions` is required.");
  }
  if (!options.filePath) {
    throw new Error("`filePath` is required.");
  }
  if (!options.project) {
    throw new Error(
      "Specify `parserOptions.project`. Otherwise there is no point in using this parser."
    );
  }
  for (const project of Array.isArray(options.project) ? options.project : [options.project]) {
    yield {
      project,
      filePath: options.filePath,
      extraFileExtensions: options.extraFileExtensions || DEFAULT_EXTRA_FILE_EXTENSIONS
    };
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  parseForESLint
});
